require 'home-dotenv/load'
require 'thor'
require 'drone/builds/parser'

# Command-line interface configuration
class Drone::Builds::Cli < Thor
  package_name 'drone-builds'

  LIMIT = Drone::Builds::DEFAULT_LIMIT

  map 'l' => :list
  map 's' => :show
  map 'v' => :version
  map 'r' => :restart
  map 'k' => :kill

  desc 'list', 'list last builds'
  method_option :limit, type: :numeric, default: LIMIT, aliases: '-l', desc: 'number of builds shown'
  method_option :pull, type: :numeric, aliases: '-p', desc: 'pull request ID on GitHub'
  method_option :branch, type: :string, aliases: '-b', desc: 'builds on that branch'
  method_option :status, type: :string, aliases: '-s', desc: 'success / failure / running / killed'
  def list
    ref = self.class.options_to_ref(options)

    builds = self.class.parser.parse_builds(self.class.client.list(
      limit: options[:limit],
      ref: ref,
      status: options[:status]
    ))

    puts self.class.parser.table(builds, %i(number status author ref message))
  end

  desc 'show', 'show a build'
  method_option :filter, type: :string, aliases: '-f', desc: 'filter builds output lines'
  method_option :context, type: :numeric, aliases: '-c', desc: 'number of lines after filter match'
  method_option :last, type: :numeric, default: 1, aliases: '-l', desc: 'show nth last build'
  method_option :pull, type: :numeric, aliases: '-p', desc: 'show last build from pull request ID'
  method_option :branch, type: :string, aliases: '-b', desc: 'builds on that branch'
  method_option :status, type: :string, aliases: '-s', desc: 'success / failure / running / killed'
  def show(build_number = nil)
    ref = self.class.options_to_ref(options)

    if build_number.to_i.to_s != build_number
      build_number = self.class.client.list(
        limit: options[:last]&.abs,
        ref: ref,
        status: options[:status]
      )[0]&.[]('number')
    end

    abort "No build found for options: #{options}" unless build_number

    self.class.client.show(
      build_number,
      filter: options[:filter],
      context: options[:context]) do |line|
      puts line
    end
  end

  desc 'restart', 'restart a build'
  def restart(build_number = nil)
    puts self.class.client.restart(build_number)
  end

  desc 'kill', 'kill a build'
  def kill(build_number = nil)
    puts self.class.client.kill(build_number)
  end

  desc 'version', 'version used'
  def version
    puts Drone::Builds::VERSION
  end

  private

  def self.parser
    @parser ||= Drone::Builds::Parser.new(client)
  end

  def self.client
    server = HOME_ENV[:DRONE_SERVER]
    domain = HOME_ENV[:DRONE_DOMAIN]

    abort("You can't specify both DRONE_SERVER and DRONE_DOMAIN") if server && domain

    if server
      protocol, domain = server.split("://")
    else
      protocol, domain = 'https', domain
    end

    @client ||=
      Drone::Builds.new(
        protocol: protocol,
        domain: domain,
        token: HOME_ENV[:DRONE_TOKEN],
        owner: HOME_ENV[:DRONE_OWNER],
        name: HOME_ENV[:DRONE_NAME],
        job_id: HOME_ENV[:DRONE_JOB_ID],
      )
  end

  def self.options_to_ref(options)
    ref = nil
    ref = "refs/heads/#{options[:branch]}" if options[:branch]
    ref = "refs/pull/#{options[:pull]}/head" if options[:pull]
    ref
  end
end
